# -*- perl -*-
#
# Phone::ReverseLookup
#
# (c)2008 Matthias Eckert
#
# Version 0.1   (2008-07-24)
#
# For history/changelog see bottom of this file.
#
#############################################################


package Phone::ReverseLookup;

use strict;
use warnings;

use constant { TRUE    => 1,
               FALSE   => 2,
               PARPROB => 3,
               ERROR   => 4
             };

# some defaults
my $debug = 0;
my $swapresults = 0;
my $baseurl = 'http://www1.dasoertliche.de/?form_name=search_inv&ph={NUMBER}';
my $regex = 'Treffer.*?title\=\"Details zu diesem Eintrag anzeigen\" class\=\"entry\" onmouseover\=\".*?\>(.*?)\<\/a\>.*?\<\/div\>(.*?)\<input type\=\"hidden\" name\=\"notepadItemsHitList\"';
my $regex_nohit = 'Kein Teilnehmer gefunden';
my $phonebook = "phonebook.tdb";
my $savehits = 1;
my $timeout = 5;
my $useragent = "Mozilla/5.0 (X11; U; Linux i686; de; rv:1.9) Gecko/2008061015 Firefox/3.0";



sub new {
    my $class = shift;
    my %param = @_;
    my $self = {};
    
    if (defined ($param{'debug'}) && $param{'debug'} =~ /^[\d]$/) {
        $self->{'debug'} = $param{'debug'};
    }
    else {
        $self->{'debug'} = $debug;
    }

    if (defined ($param{'swapResults'}) && $param{'swapResults'} =~ /^[\d]$/) {
        $self->{'swapResults'} = $param{'swapResults'};
    }
    else {
        $self->{'swapResults'} = $swapresults;
    }
    
    if (defined ($param{'baseURL'}) && $param{'baseURL'}) {
        $self->{'baseURL'} = $param{'baseURL'};
    }
    else {
        $self->{'baseURL'} = $baseurl;
    }
    $self->{'baseURL'} =~ /^(.*)$/;
    $self->{'baseURL'} = $1;
    
    if (defined ($param{'regEx'}) && $param{'regEx'}) {
        $self->{'regEx'} = $param{'regEx'};
    }
    else {
        $self->{'regEx'} = $regex;
    }

    if (defined ($param{'regExNoHit'}) && $param{'regExNoHit'}) {
        $self->{'regExNoHit'} = $param{'regExNoHit'};
    }
    else {
        $self->{'regExNoHit'} = $regex_nohit;
    }
    
    if (defined ($param{'phoneBook'}) && $param{'phoneBook'}) {
        $self->{'phoneBook'} = $param{'phoneBook'};
    }
    else {
        $self->{'phoneBook'} = $phonebook;
    }
    $self->{'phoneBook'} =~ /^(.*)$/;
    $self->{'phoneBook'} = $1;

    if (defined ($param{'saveHits'}) && $param{'saveHits'} =~ /^[\d]$/) {
        $self->{'saveHits'} = $param{'saveHits'};
    }
    else {
        $self->{'saveHits'} = $savehits;
    }
    
    if (defined ($param{'timeOut'}) && $param{'timeOut'}) {
        $self->{'timeOut'} = $param{'timeOut'};
    }
    else {
        $self->{'timeOut'} = $timeout;
    }
    $self->{'timeOut'} =~ /^([\d]+)$/;
    $self->{'timeOut'} = $1;

    if (defined ($param{'userAgent'}) && $param{'userAgent'}) {
        $self->{'userAgent'} = $param{'userAgent'};
    }
    else {
        $self->{'userAgent'} = $useragent;
    }
    $self->{'userAgent'} =~ /^(.*)$/;
    $self->{'userAgent'} = $1;
    
    bless ($self, $class);
    
    return (TRUE, $self);
}



sub find_executable {
    my $self = shift;
    my $executableName = shift;
    my $dir;
    $ENV{PATH} = "/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin";
    my @path = split(/:/, $ENV{PATH});
    
    if (! defined ($executableName) || ! $executableName) {
        $self->{'debug'} && print STDERR "find_executable: Invalid number of arguments\n";
        return (PARPROB, "find_executable: Invalid number of arguments");
    }
    
    foreach $dir (@path) {
        (-x "$dir/$executableName") and return (TRUE, "$dir/$executableName");
    }
    
    $self->{'debug'} && print STDERR "find_executable: Program '$executableName' not found\n";
    return (FALSE, "find_executable: Program '$executableName' not found");
}



sub add_to_phonebook {    
    my $self = shift;
    my %param = @_;
    my ($status, $msg);
#    my %old;

    if (defined $param{'number'} && $param{'number'} && length($param{'number'}) > 1 && $param{'number'} =~ /^[\d]+$/) {
        $self->{'number'} = $param{'number'};
    }
    if (defined $param{'name'} && $param{'name'}) {
        $self->{'name'} = $param{'name'};
    }
    if (defined $param{'detail'} && $param{'detail'}) {
        $self->{'detail'} = $param{'detail'};
    }
    if (defined $param{'timestamp'} && $param{'timestamp'}) {
        $self->{'timestamp'} = $param{'timestamp'};
    }

    if (! defined ($self->{'number'}) || ! $self->{'number'} || length($self->{'number'}) < 2 || $self->{'number'} !~ /^[\d]+$/) {
        $self->{'debug'} && print STDERR "add_to_phonebook: Invalid number\n";
        return (PARPROB, "add_to_phonebook: Invalid number");
    }
    if (! defined ($self->{'name'}) || ! $self->{'name'}) {
        $self->{'debug'} && print STDERR "add_to_phonebook: Invalid name\n";
        return (PARPROB, "add_to_phonebook: Invalid name");
    }
    if (! defined ($self->{'detail'}) || ! $self->{'detail'}) {
        $self->{'detail'} = "";
    }
    if (! defined ($self->{'timestamp'}) || ! $self->{'timestamp'}) {
        $self->{'timestamp'} = time();
    }
    
#    $old{'number'} = $self->{'number'};
#    $old{'name'} = $self->{'name'};
#    $old{'detail'} = $self->{'detail'};
#    $old{'timestamp'} = $self->{'timestamp'};
    
    ($status, $msg) = $self->phonebook_lookup();
    if ($status == FALSE) {
        if (open (PB, ">> $self->{'phoneBook'}")) {
            print PB "$self->{'number'}|$self->{'name'}|$self->{'detail'}|$self->{'timestamp'}\n";
            close PB;
            return (TRUE, $self);
        }
        else {
            $self->{'debug'} && print STDERR "add_to_phonebook: Can't open phone book file '$self->{'phoneBook'}'\n";
            return (ERROR, "add_to_phonebook: Can't open phone book file '$self->{'phoneBook'}'");
        }
    }
    elsif ($status == TRUE) {
        $self->{'debug'} && print STDERR "add_to_phonebook: Number already exists\n";
        return (FALSE, "add_to_phonebook: Number already exists");
    }
    else {
        $self->{'debug'} && print STDERR "$msg\n";
        return ($status, $msg);
    }
}



sub phonebook_lookup {
    my $self = shift;
    my %param = @_;
    my ($num, $name, $detail, $timestamp);
    my $timenow = time();
    my $diff = 0;
    
    if (defined $param{'number'} && $param{'number'} && length($param{'number'}) > 1 && length($param{'number'}) < 21 && $param{'number'} =~ /^[\d]+$/) {
        $self->{'number'} = $param{'number'};
    }

    if (! defined ($self->{'number'}) || ! $self->{'number'} || length($self->{'number'}) < 2 || length($self->{'number'}) > 20 || $self->{'number'} !~ /^[\d]+$/) {
        $self->{'debug'} && print STDERR "phonebook_lookup: Invalid number\n";
        return (PARPROB, "phonebook_lookup: Invalid number");
    }

    # needs work !!!
    if ($self->{'number'} =~ /^\+49/ || $self->{'number'} =~ /^0049/ || $self->{'number'} =~ /^49/) {
        $self->{'number'} =~ s/^(\+49|0049|49)/0/;
        $self->{'number'} =~ s/^[0]{2,}/0/;
    }

    if (! -f $self->{'phoneBook'} || ! -r $self->{'phoneBook'}) {
        $self->{'debug'} && print STDERR "phonebook_lookup: Phone book file '$self->{'phoneBook'}' not found or not readable\n";
        return (ERROR, "phonebook_lookup: Phone book file '$self->{'phoneBook'}' not found or not readable");
    }
    
    $self->{'isLocalHit'} = 0;

    if (open (PB, "$self->{'phoneBook'}")) {
        while (<PB>) {
            s/[\r\n]+//;
            next if (! /^[\d]+/);
            ($num, $name, $detail, $timestamp) = split (/\|/, $_, 4);
            if (defined ($timestamp) && $timestamp && $timestamp =~ /^[\d]+$/) {
                $diff = int($timenow - $timestamp);
            }
            if (defined ($num) && $num && $num eq $self->{'number'} && defined ($name) && $name) {
                close PB;
                $self->{'isLocalHit'} = 1;
                $self->{'name'} = $name;
                if (defined ($detail) && $detail) {
                    $self->{'detail'} = $detail;
                }
                if (defined ($timestamp) && $timestamp) {
                    $self->{'timestamp'} = $timestamp;
                }
                return (TRUE, $self);
            }
        }
        close PB;
    }
    else {
        $self->{'debug'} && print STDERR "phonebook_lookup: Can't open phone book file '$self->{'phoneBook'}'\n";
        return (ERROR, "phonebook_lookup: Can't open phone book file '$self->{'phoneBook'}'");
    }
    $self->{'debug'} && print STDERR "phonebook_lookup: No match for number '$self->{'number'}'\n";
    return (FALSE, "phonebook_lookup: No match for number '$self->{'number'}'");
}



sub online_lookup {
    my $self = shift;
    my %param = @_;
    my ($num, $name, $detail);
    my ($status, $msg);
    my $url;
    my $regex;
    my $content;

    if (defined $param{'number'} && $param{'number'} && length($param{'number'}) > 6 && length($param{'number'}) < 21 && $param{'number'} =~ /^[\d]+$/) {
        $self->{'number'} = $param{'number'};
    }

    if (! defined ($self->{'number'}) || ! $self->{'number'} || length($self->{'number'}) < 7 || length($self->{'number'}) > 20 || $self->{'number'} !~ /^[\d]+$/) {
        $self->{'debug'} && print STDERR "online_lookup: Invalid number\n";
        return (PARPROB, "online_lookup: Invalid number");
    }
        
    # needs work !!!
    if ($self->{'number'} =~ /^\+49/ || $self->{'number'} =~ /^0049/ || $self->{'number'} =~ /^49/) {
        $self->{'number'} =~ s/^(\+49|0049|49)/0/;
        $self->{'number'} =~ s/^[0]{2,}/0/;
    }

    $url = $self->{'baseURL'};
    $regex = $self->{'regEx'};
    if ($url =~ /\{NUMBER\}/) {
        $url =~ s/^(.*?)\{NUMBER\}(.*?)$/$1$self->{'number'}$2/;
    }
    if ($regex =~ /\{NUMBER\}/) {
        $regex =~ s/^(.*?)\{NUMBER\}(.*?)$/$1$self->{'number'}$2/;
    }    
    
    $self->{'isOnlineHit'} = 0;
    
    if (! defined ($content) || ! $content) {
        ($status, $msg) = $self->try_wget($url);
        if ($status == TRUE) {
            $content = $msg;
        }
        else {
            $self->{'debug'} && print STDERR "$msg\n";
        }
    }
    
    if (! defined ($content) || ! $content) {
        ($status, $msg) = $self->try_lynx($url);
        if ($status == TRUE) {
            $content = $msg;
        }
        else {
            $self->{'debug'} && print STDERR "$msg\n";
        }
    }
    
    if (! defined ($content) || ! $content) {
        ($status, $msg) = $self->try_curl($url);
        if ($status == TRUE) {
            $content = $msg;
        }
        else {
            $self->{'debug'} && print STDERR "$msg\n";
        }
    }

    if (defined ($content) && $content) {
        $content =~ s/[\r\n]+/\ /g;
        $content =~ s/[\x00-\x1F]+//g;
#        $content =~ s/[\x7F-\xFF]+//g;
        $content =~ s/\&nbsp\;/\ /g;
#        $content =~ s/\<br\/?\>/\ /g;
        $content =~ s/[\s\t]+/\ /g;
        if (defined ($content) && $content) {
            if (defined ($self->{'regExNoHit'}) && $self->{'regExNoHit'} && $content =~ m/$self->{'regExNoHit'}/i) {
                $self->{'debug'} && print STDERR "online_lookup: No match for number '$self->{'number'}'\n";
                return (FALSE, "online_lookup: No match for number '$self->{'number'}'");
            }
            elsif ($content =~ m/$regex/i) {
                if ($self->{'swapResults'}) {
                    $detail = $1;
                    $name = $2;
                }
                else {
                    $name = $1;
                    $detail = $2;
                }
                if (defined ($name) && $name) {
                    $name =~ s/^[\s\t]+//;
                    $name =~ s/[\s\t]+$//;
                    $name =~ s/^[\!]+//;
                    $name =~ s/^\<.*?\>//g;
                    $name =~ s/\<.*?\>$//g;
                    $name =~ s/\<.*?\>/\,\ /g;
                    $name =~ s/^[\s\t]+//;
                    $name =~ s/[\s\t]+$//;
                    $name =~ s/[\,]{2,}/\,/;
                    $name =~ s/[\.]{2,}/\./;
                    $name =~ s/[\,]+$//;
                    $name =~ s/\&([aou])uml\;/$1e/g;
                    $name =~ s/\&szlig\;/ss/g;
                }
                if (defined ($detail) && $detail) {
                    $detail =~ s/^[\s\t]+//;
                    $detail =~ s/[\s\t]+$//;
                    $detail =~ s/^[\!]+//;
                    $detail =~ s/^\<.*?\>//g;
                    $detail =~ s/\<.*?\>$//g;
                    $detail =~ s/\<.*?\>/\,\ /g;
                    $detail =~ s/^[\s\t]+//;
                    $detail =~ s/[\s\t]+$//;
                    $detail =~ s/[\,]{2,}/\,/;
                    $detail =~ s/[\.]{2,}/\./;
                    $detail =~ s/[\,]+$//;
                    $detail =~ s/\&([aou])uml\;/$1e/g;
                    $detail =~ s/\&szlig\;/ss/g;
                }
                if (defined ($name) && $name) {
                    $self->{'isOnlineHit'} = 1;
                    $self->{'name'} = substr ($name, 0, 128);
                    if (defined ($detail) && $detail) {
                        $self->{'detail'} = substr ($detail, 0, 256);
                    }
                    if ($self->{'saveHits'}) {
                        ($status, $msg) = $self->add_to_phonebook();
                        if ($status != TRUE) {
                            $self->{'debug'} && print STDERR "$msg\n";
                        }
                    }
                    return (TRUE, $self);
                }
            }
        }
        else {
            $self->{'debug'} && print STDERR "online_lookup: No content\n";
            return (ERROR, "online_lookup: No content");
        }
    }
    else {
        $self->{'debug'} && print STDERR "online_lookup: Request failed\n";
        return (ERROR, "online_lookup: Request failed");
    }
    $self->{'debug'} && print STDERR "online_lookup: No match for number '$self->{'number'}'\n";
    return (FALSE, "online_lookup: No match for number '$self->{'number'}'");
}



sub try_wget {
    my $self = shift;
    my $url = shift;
    my $wget;
    my $cmd;
    my $oldfs = $/;
    my ($status, $msg);
    my $content;
    
    if (! defined ($url) || ! $url) {
        $self->{'debug'} && print STDERR "try_wget: Invalid url\n";
        return (PARPROB, "try_wget: Invalid url");
    }
    
    ($status, $msg) = $self->find_executable("wget");
    if ($status == TRUE) {
        $wget = $msg;
        $cmd = "$wget -q --timeout=$self->{'timeOut'} --user-agent=\"$self->{'userAgent'}\" \"$url\" -O - 2>/dev/null >&1 |";
        $cmd =~ /^(.*)$/;
        $cmd = $1;
        if (open (FH, $cmd)) {
#        if (open (FH, "$wget -q --timeout=$self->{'timeOut'} --user-agent=\"$self->{'userAgent'}\" \"$url\" -O - 2>/dev/null >&1 |")) {
            $/ = undef;
            $content = <FH>;
            $/ = $oldfs;
            close FH;
            if (defined ($content) && $content) {
                return (TRUE, $content);
            }
        }
        else {
            $self->{'debug'} && print STDERR "try_wget: Couldn't run wget command\n";
            return (ERROR, "try_wget: Couldn't run wget command");
        }
    }
    else {
        $self->{'debug'} && print STDERR "$msg\n";
        return ($status, $msg);
    }
    $self->{'debug'} && print STDERR "try_wget: Request failed\n";
    return (ERROR, "try_wget: Request failed");
}



sub try_lynx {
    my $self = shift;
    my $url = shift;
    my $lynx;
    my $cmd;
    my $oldfs = $/;
    my ($status, $msg);
    my $content;
    
    if (! defined ($url) || ! $url) {
        $self->{'debug'} && print STDERR "try_lynx: Invalid url\n";
        return (PARPROB, "try_lynx: Invalid url");
    }
    
    ($status, $msg) = $self->find_executable("lynx");
    if ($status == TRUE) {
        $lynx = $msg;
        $cmd = "$lynx -connect_timeout=$self->{'timeOut'} -useragent=\"$self->{'userAgent'}\" \"$url\" -source 2>/dev/null >&1 |";
        $cmd =~ /^(.*)$/;
        $cmd = $1;        
        if (open (FH, $cmd)) {
#        if (open (FH, "$lynx -connect_timeout=$self->{'timeOut'} -useragent=\"$self->{'userAgent'}\" \"$url\" -source 2>/dev/null >&1 |")) {
            $/ = undef;
            $content = <FH>;
            $/ = $oldfs;
            close FH;
            if (defined ($content) && $content) {
                return (TRUE, $content);
            }
        }
        else {
            $self->{'debug'} && print STDERR "try_lynx: Couldn't run lynx command\n";
            return (ERROR, "try_lynx: Couldn't run lynx command");
        }
    }
    else {
        $self->{'debug'} && print STDERR "$msg\n";
        return ($status, $msg);
    }
    $self->{'debug'} && print STDERR "try_lynx: Request failed\n";
    return (ERROR, "try_lynx: Request failed");
}



sub try_curl {
    my $self = shift;
    my $url = shift;
    my $curl;
    my $cmd;
    my $oldfs = $/;
    my ($status, $msg);
    my $content;
    
    if (! defined ($url) || ! $url) {
        $self->{'debug'} && print STDERR "try_curl: Invalid url\n";
        return (PARPROB, "try_curl: Invalid url");
    }
    
    ($status, $msg) = $self->find_executable("curl");
    if ($status == TRUE) {
        $curl = $msg;
        $cmd = "$curl -s --connect-timeout $self->{'timeOut'} --user-agent \"$self->{'userAgent'}\" \"$url\" -o - 2>/dev/null >&1 |";
        $cmd =~ /^(.*)$/;
        $cmd = $1;        
        if (open (FH, $cmd)) {
#        if (open (FH, "$curl -s --connect-timeout $self->{'timeOut'} --user-agent \"$self->{'userAgent'}\" \"$url\" -o - 2>/dev/null >&1 |")) {
            $/ = undef;
            $content = <FH>;
            $/ = $oldfs;
            close FH;
            if (defined ($content) && $content) {
                return (TRUE, $content);
            }
        }
        else {
            $self->{'debug'} && print STDERR "try_curl: Couldn't run curl command\n";
            return (ERROR, "try_curl: Couldn't run curl command");
        }
    }
    else {
        $self->{'debug'} && print STDERR "$msg\n";
        return ($status, $msg);
    }
    $self->{'debug'} && print STDERR "try_curl: Request failed\n";
    return (ERROR, "try_curl: Request failed");
}




sub lookup_old {
# don't use !!!
return 0;
    my $self = shift;
    my %param = @_;
    my ($status, $msg);
    
    if (defined $param{'number'} && $param{'number'} && length($param{'number'}) > 1 && length($param{'number'}) < 21 && $param{'number'} =~ /^\+?[\d]+$/) {
        $self->{'number'} = $param{'number'};
    }

    if (! defined ($self->{'number'}) || ! $self->{'number'} || length($self->{'number'}) < 2 || length($self->{'number'}) > 20 || $self->{'number'} !~ /^\+?[\d]+$/) {
        $self->{'debug'} && print STDERR "lookup: Invalid number\n";
        return (PARPROB, "lookup: Invalid number");
    }
    
    # needs work !!!
    if ($self->{'number'} =~ /^\+49/ || $self->{'number'} =~ /^0049/ || $self->{'number'} =~ /^49/) {
        $self->{'number'} =~ s/^(\+49|0049|49)/0/;
        $self->{'number'} =~ s/^[0]{2,}/0/;
    }
    
    $self->{'isLocalHit'} = 0;
    $self->{'isOnlineHit'} = 0;

    ($status, $msg) = $self->phonebook_lookup();
    if ($status != TRUE) {
        $self->{'debug'} && print STDERR "$msg\n";
        ($status, $msg) = $self->online_lookup();
        if ($status != TRUE) {
            $self->{'debug'} && print STDERR "$msg\n";
            return ($status, $msg);
        }
        else {
            if ($self->{'saveHits'}) {
                ($status, $msg) = $self->add_to_phonebook();
                if ($status != TRUE) {
                    $self->{'debug'} && print STDERR "$msg\n";
                }
            }
            return (TRUE, $self);
        }
    }
    return (TRUE, $self);
}



1;
#############################################################
#
# History:
#
# Version 0.1 (2008-07-24)
# - initial version
#
#############################################################