#!/usr/bin/perl -w
#
# reverse-lookup.pl
#
# (c)2008 Matthias Eckert
#
# Version 0.1   (2008-07-24)
#
# For history/changelog see bottom of this file.
#
#############################################################

use strict;
use warnings;

use lib "./lib";

use Phone::ReverseLookup;

use constant { TRUE    => 1,
               FALSE   => 2,
               PARPROB => 3,
               ERROR   => 4
            };


my %Config = (
		'www.11880.com'		=> { 'baseURL'		=> 'http://www.11880.com/Suche/index.cfm?fuseaction=Suche.rueckwaertssucheresult&init=true&change=false&searchform=Rueckwaerts&tel={NUMBER}',
				    	     'regEx'		=> '.*?href\=\"\/detailseiten\/.*?title\=\"Alle Details anzeigen\"\>(.*?)\<\/a\>.*?\<div style\=\"padding\-bottom\:5px\;\"\>(.*?)\<br\s?\/\>.*',
				    	     'regExNoHit'	=> 'Es wurden keine Treffer gefunden',
				    	     'swapResults'	=> 0 },
                                	
                'www.dasoertliche.de'	=> { 'baseURL'          => 'http://www1.dasoertliche.de/?form_name=search_inv&ph={NUMBER}',
                                    	     'regEx'            => 'Treffer.*?title\=\"Details zu diesem Eintrag anzeigen\" class\=\"entry\" onmouseover\=\".*?\>(.*?)\<\/a\>.*?\<\/div\>(.*?)\<input type\=\"hidden\" name\=\"notepadItemsHitList\"',
				    	     'regExNoHit'	=> 'Kein Teilnehmer gefunden',
                                    	     'swapResults'	=> 0 },
                                    	
                'www.meinestadt.de'	=> { 'baseURL'          => 'http://www.meinestadt.de/deutschland/telefonbuch/suche?number={NUMBER}&formtype=rueckwaertssuche&Submit=suchen',
                                    	     'regEx'            => '\<a class\=\"katalogtitel\" href\=\".*?\"\>(.*?)\<\/a\>\<br\>(.*?)\<b\>Tel\.:.*?',
				    	     'regExNoHit'	=> 'Leider keine Treffer',
                                    	     'swapResults'	=> 0 },

                'www.goyellow.de'	=> { 'baseURL'          => 'http://www.goyellow.de/inverssuche/?TEL={NUMBER}',
                                    	     'regEx'            => '\<a href\=\"\/upgrade.*?\" title\=\"Detailinformationen zu .*?\"\>(.*?)\<\/a\>.*?\<p class\=\"address\"\>(.*?)\<\/p\>',
				    	     'regExNoHit'	=> 'Es wurden keine Eintr.*?ge gefunden',
                                    	     'swapResults'	=> 0 },

                'www.klicktel.de'	=> { 'baseURL'          => 'http://www.klicktel.de/inverssuche/backwardssearch.html?newSearch=1&vollstaendig={NUMBER}',
                                    	     'regEx'            => 'Ihre Suche nach.*?haben wir 1 Treffer gefunden.*?\<ul class\=\"listenansicht\"\>.*?\<a class\=\"head\" href\=\"view\.html.*?\>\<span class\=\"title\"\>(.*?)\<\/span\>\<\/a\>.*?\<span class\=\"location\"\>(.*?)\<\/span\>',
				    	     'regExNoHit'	=> '(Zu Ihrer Suche wurden leider keine Ergebnisse gefunden|Die Ergebnisse wurden durch Verk.*?rzung der gesuchten Rufnummer auf.*?gefunden)',
                                    	     'swapResults'	=> 0 },
                                    	
                'FritzBox'		=> { 'baseURL'          => 'http://fritz.box/cgi-bin/webcm?getpage=..%2Fhtml%2Fde%2Fmenus%2Fmenu2.html&errorpage=..%2Fhtml%2Fde%2Fmenus%2Fmenu2.html&var%3Alang=de&var%3Apagename=fonbuch&var%3Amenu=home',
                                    	     'regEx'            => '.*\<script type\=\"text\/javascript\"\>document\.write\(TrFon\(\"\d+\"\, \"(.*?)\"\, \"{NUMBER}\"\, \".*?\"\, \".*?\"\)\)\;\<\/script\>',
				    	     'regExNoHit'	=> undef,
                                    	     'swapResults'	=> 0 }
	    );



### MAIN ###

    my $number = $ARGV[0];
    my $obj;
    my ($status, $msg);
    my $debug = 0;
    my $savehits = 1;
    my $timeout = 5;
    my $config;
    
    
    if (defined ($number) && $number && $number =~ /\+?[\d]+/) {
        ($status, $msg) = Phone::ReverseLookup->new("phoneBook" => "phonebook.tdb", "saveHits" => $savehits, "debug" => $debug, "timeOut" => $timeout);
        if ($status == TRUE) {
            $obj = $msg;
            ($status, $msg) = $obj->phonebook_lookup("number" => "$number");
            if ($status == TRUE) {
                print STDOUT "SOURCE: Lokales Adressbuch\n";
                print STDOUT "NUMBER: $obj->{'number'}\n";
                print STDOUT "NAME: $obj->{'name'}\n";
                if (defined ($obj->{'detail'}) && $obj->{'detail'}) {
                    print STDOUT "DETAIL: $obj->{'detail'}\n";
                }
            }
            elsif ($status == FALSE) {
                $debug && print STDOUT "DEBUG: Konnte Teilnehmer nicht im lokalen Adressbuch finden\n";
                foreach $config (keys %Config) {
                    $obj->{'baseURL'} = $Config{$config}{'baseURL'};
                    $obj->{'regEx'} = $Config{$config}{'regEx'};
                    $obj->{'regExNoHit'} = $Config{$config}{'regExNoHit'};
                    $obj->{'swapResults'} = $Config{$config}{'swapResults'};
                    ($status, $msg) = $obj->online_lookup("number" => "$number");
                    if ($status == TRUE) {
                        print STDOUT "SOURCE: $config\n";
                        print STDOUT "NUMBER: $obj->{'number'}\n";
                        print STDOUT "NAME: $obj->{'name'}\n";
                        if (defined ($obj->{'detail'}) && $obj->{'detail'}) {
                            print STDOUT "DETAIL: $obj->{'detail'}\n";
                        }
                        last;
                    }
                    elsif ($status == FALSE) {
                        $debug && print STDOUT "DEBUG: Konnte Teilnehmer online nicht finden\n";
                        next;
                    }
                    else {
                        $debug && print STDOUT "DEBUG: $msg\n";
                    }
                }
            }
            else {
                $debug && print STDOUT "DEBUG: $msg\n";
            }
        }
        else {
            $debug && print STDOUT "DEBUG: $msg\n";
        }    
    }
    else {
        print STDOUT "ERROR: Konnte im uebergebenen Argument keine Nummer erkennen\n";
        exit 1;
    }
    
    exit 0;


#############################################################
#
# History:
#
# Version 0.1 (2008-07-24)
# - initial version
#
#############################################################